"""
Some simple settings for use in other parts of the tool.
"""

import os

HOME = os.environ['HOME']
PYHOME_ROOT = os.path.join(HOME, '.config/pyhome')
PYHOME_REPO = os.path.join(PYHOME_ROOT, 'repos')
